@@ -1,6 +1,6 @@
 '''cr_tests.py: common setup and tests for test modules'''
 #
-# Copyright (C) 2013-2014 Canonical Ltd.
+# Copyright (C) 2013-2015 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ import json
 import os
 import tempfile
 from xdg.DesktopEntry import DesktopEntry
+import yaml
 
 from unittest.mock import patch
 from unittest import TestCase
@@ -29,7 +30,10 @@ import clickreviews.cr_common as cr_common
 # These should be set in the test cases
 TEST_CONTROL = ""
 TEST_MANIFEST = ""
+TEST_PKG_YAML = ""
+TEST_README_MD = ""
 TEST_SECURITY = dict()
+TEST_SECURITY_PROFILES = dict()
 TEST_DESKTOP = dict()
 TEST_WEBAPP_MANIFESTS = dict()
 TEST_URLS = dict()
@@ -63,6 +67,16 @@ def _extract_manifest_file(self):
     return io.StringIO(TEST_MANIFEST)
 
 
+def _extract_package_yaml(self):
+    '''Pretend we read the package.yaml file'''
+    return io.StringIO(TEST_PKG_YAML)
+
+
+def _extract_readme_md(self):
+    '''Pretend we read the meta/readme.md file'''
+    return TEST_README_MD
+
+
 def _extract_click_frameworks(self):
     '''Pretend we enumerated the click frameworks'''
     return ["ubuntu-sdk-13.10",
@@ -95,6 +109,16 @@ def _get_security_manifest(self, app):
     return ("%s.apparmor" % app, json.loads(TEST_SECURITY[app]))
 
 
+def _extract_security_profile(self, app):
+    '''Pretend we read the security profile'''
+    return io.StringIO(TEST_SECURITY_PROFILES[app])
+
+
+def _get_security_profile(self, app):
+    '''Pretend we read the security profile'''
+    return ("%s.profile" % app, TEST_SECURITY_PROFILES[app])
+
+
 def _get_security_supported_policy_versions(self):
     '''Pretend we read the contens of /usr/share/apparmor/easyprof'''
     return [1.0, 1.1, 1.2, 1.3]
@@ -176,7 +200,7 @@ def _extract_framework(self, app):
 
 def _extract_systemd(self, app):
     '''Pretend we found the systemd file'''
-    return ("%s.click-service" % app, TEST_SNAPPY_SYSTEMD[app])
+    return ("%s.snappy-systemd" % app, TEST_SNAPPY_SYSTEMD[app])
 
 
 # http://docs.python.org/3.4/library/unittest.mock-examples.html
@@ -200,6 +224,9 @@ patches.append(patch(
 patches.append(patch(
     'clickreviews.cr_common.ClickReview._extract_manifest_file',
     _extract_manifest_file))
+patches.append(patch(
+    'clickreviews.cr_common.ClickReview._extract_package_yaml',
+    _extract_package_yaml))
 patches.append(patch(
     'clickreviews.cr_common.ClickReview._extract_click_frameworks',
     _extract_click_frameworks))
@@ -219,6 +246,9 @@ patches.append(patch('clickreviews.cr_lint.ClickReviewLint._list_all_files',
 patches.append(patch(
     'clickreviews.cr_lint.ClickReview._list_all_compiled_binaries',
     _mock_func))
+patches.append(patch(
+    'clickreviews.cr_lint.ClickReviewLint._extract_readme_md',
+    _extract_readme_md))
 
 # security overrides
 patches.append(patch(
@@ -227,6 +257,12 @@ patches.append(patch(
 patches.append(patch(
     'clickreviews.cr_security.ClickReviewSecurity._get_security_manifest',
     _get_security_manifest))
+patches.append(patch(
+    'clickreviews.cr_security.ClickReviewSecurity._extract_security_profile',
+    _extract_security_profile))
+patches.append(patch(
+    'clickreviews.cr_security.ClickReviewSecurity._get_security_profile',
+    _get_security_profile))
 
 # desktop overrides
 patches.append(patch(
@@ -339,8 +375,19 @@ class TestClickReview(TestCase):
             "%s.url-dispatcher" % self.default_appname
         self._update_test_manifest()
 
+        self.test_pkg_yaml = dict()
+        self.set_test_pkg_yaml("name", self.test_control['Package'])
+        self.set_test_pkg_yaml("version", self.test_control['Version'])
+        self.set_test_pkg_yaml("architecture",
+                               self.test_control['Architecture'])
+        self._update_test_pkg_yaml()
+
+        self.test_readme_md = self.test_control['Description']
+        self._update_test_readme_md()
+
         # hooks
         self.test_security_manifests = dict()
+        self.test_security_profiles = dict()
         self.test_desktop_files = dict()
         self.test_url_dispatcher = dict()
         self.test_scopes = dict()
@@ -400,7 +447,11 @@ class TestClickReview(TestCase):
             # Reset to no systemd entries in manifest
             self.set_test_systemd(app, None, None)
 
+            # Reset to no security profiles
+            self.set_test_security_profile(app, None)
+
         self._update_test_security_manifests()
+        self._update_test_security_profiles()
         self._update_test_desktop_files()
         self._update_test_url_dispatcher()
         self._update_test_scopes()
@@ -431,12 +482,31 @@ class TestClickReview(TestCase):
         global TEST_MANIFEST
         TEST_MANIFEST = json.dumps(self.test_manifest)
 
+    def _update_test_pkg_yaml(self):
+        global TEST_PKG_YAML
+        TEST_PKG_YAML = yaml.dump(self.test_pkg_yaml,
+                                  default_flow_style=False,
+                                  indent=4)
+
+    def _update_test_readme_md(self):
+        global TEST_README_MD
+        TEST_README_MD = self.test_readme_md
+
     def _update_test_security_manifests(self):
         global TEST_SECURITY
         TEST_SECURITY = dict()
         for app in self.test_security_manifests.keys():
             TEST_SECURITY[app] = json.dumps(self.test_security_manifests[app])
 
+    def _update_test_security_profiles(self):
+        global TEST_SECURITY_PROFILES
+        TEST_SECURITY_PROFILES = dict()
+        for app in self.test_security_profiles.keys():
+            TEST_SECURITY_PROFILES[app] = self.test_security_profiles[app]
+            self.test_manifest["hooks"][app]["apparmor-profile"] = \
+                "%s.profile" % app
+        self._update_test_manifest()
+
     def _update_test_desktop_files(self):
         global TEST_DESKTOP
         TEST_DESKTOP = dict()
@@ -552,6 +622,9 @@ class TestClickReview(TestCase):
         TEST_SNAPPY_SYSTEMD = dict()
         for app in self.test_systemd.keys():
             TEST_SNAPPY_SYSTEMD[app] = self.test_systemd[app]
+            self.test_manifest["hooks"][app]["snappy-systemd"] = \
+                "%s.snappy-systemd" % app
+        self._update_test_manifest()
 
     def _update_test_name(self):
         self.test_name = "%s_%s_%s.click" % (self.test_control['Package'],
@@ -619,6 +692,24 @@ class TestClickReview(TestCase):
             self.test_manifest[key] = value
         self._update_test_manifest()
 
+    def set_test_pkg_yaml(self, key, value):
+        '''Set key in meta/package.yaml to value. If value is None, remove
+           key'''
+        if value is None:
+            if key in self.test_pkg_yaml:
+                self.test_pkg_yaml.pop(key, None)
+        else:
+            self.test_pkg_yaml[key] = value
+        self._update_test_pkg_yaml()
+
+    def set_test_readme_md(self, contents):
+        '''Set contents of meta/readme.md'''
+        if contents is None:
+            self.test_readme_md = None
+        else:
+            self.test_readme_md = contents
+        self._update_test_readme_md()
+
     def set_test_security_manifest(self, app, key, value):
         '''Set key in security manifest to value. If value is None, remove
            key'''
@@ -632,6 +723,17 @@ class TestClickReview(TestCase):
             self.test_security_manifests[app][key] = value
         self._update_test_security_manifests()
 
+    def set_test_security_profile(self, app, policy):
+        '''Set policy in security profile'''
+        if policy is None:
+            if app in self.test_security_profiles:
+                self.test_security_profiles.pop(app)
+        else:
+            if app not in self.test_security_profiles:
+                self.test_security_profiles[app] = dict()
+            self.test_security_profiles[app] = policy
+        self._update_test_security_profiles()
+
     def set_test_desktop(self, app, key, value, no_update=False):
         '''Set key in desktop file to value. If value is None, remove key'''
         if app not in self.test_desktop_files:
@@ -774,17 +876,21 @@ class TestClickReview(TestCase):
             self.test_framework[app][key] = value
         self._update_test_framework()
 
-    def set_test_systemd(self, app, key, value, append=False):
-        '''Set systemd entries. If value is None, remove'''
-        if app not in self.test_systemd:
-            self.test_systemd[app] = []
-
-        if value is None:
-            self.test_systemd[app] = []
+    def set_test_systemd(self, app, key, value):
+        '''Set systemd entries. If key is None, remove hook, if value is None,
+           remove key'''
+        if key is None:
+            if app in self.test_systemd:
+                self.test_systemd.pop(app)
         else:
-            if not append:
-                self.test_systemd[app] = []
-            self.test_systemd[app].append({key: value})
+            if app not in self.test_systemd:
+                self.test_systemd[app] = {}
+
+            if value is None:
+                if key in self.test_systemd[app]:
+                    del(self.test_systemd[app][key])
+            else:
+                self.test_systemd[app][key] = value
         self._update_test_systemd()
 
     def setUp(self):
@@ -799,8 +905,14 @@ class TestClickReview(TestCase):
         TEST_CONTROL = ""
         global TEST_MANIFEST
         TEST_MANIFEST = ""
+        global TEST_PKG_YAML
+        TEST_PKG_YAML = ""
+        global TEST_README_MD
+        TEST_README_MD = ""
         global TEST_SECURITY
         TEST_SECURITY = dict()
+        global TEST_SECURITY_PROFILES
+        TEST_SECURITY_PROFILES = dict()
         global TEST_DESKTOP
         TEST_DESKTOP = dict()
         global TEST_URLS
